/* 
 */

module crc32
(
  input clk, reset,
  input [7:0] data_in,
  input data_wr,
  input crc_reg_in, crc_reg_ld, crc_reg_wr,
  input lut_reg_in, lut_reg_wr,
  output data_out,
  output crc_reg_out,
  output [7:0] lut_addr
);
  wire clk, reset; // standard signals
  wire [7:0] data_in; // data input for the CRC calc
  wire data_wr;    // data register write
  wire crc_reg_in; // crc_reg shift in
  wire crc_reg_wr; // crc_reg write signal
  wire crc_reg_ld; // crc_reg direct load
  wire lut_reg_in; // crc_reg direct load
  wire lut_reg_wr; // crc_reg write signal

  // internal wires
  wire [7:0] data_out_bus;
  wire [31:0] crc_reg_in_bus;
  wire [31:0] crc_reg_out_bus;
  wire [31:0] lut_reg_out_bus;

  sr #(8) data_reg(
    .clk(clk),
    .reset(reset),
    .ce(data_wr),
    .load(1),
    .din(0),
    .din_load(data_in),
    .dout(data_out_bus)
  );

  sr #(32) crc_reg(
    .clk(clk),
    .reset(reset),
    .ce(crc_reg_wr),
    .load(crc_reg_ld),
    .din(~crc_reg_in), // make sure the input is inverted
    .din_load(crc_reg_in_bus),
    .dout(crc_reg_out_bus)
  );

  sr #(32) lut_reg(
    .clk(clk),
    .reset(reset),
    .ce(lut_reg_wr),
    .load(0),
    .din(lut_reg_in),
    .din_load(0),
    .dout(lut_reg_out_bus)
  );

  assign lut_addr = data_out_bus ^ crc_reg_out_bus[7:0];
  assign crc_reg_in_bus =  {8'b0, crc_reg_out_bus[31:8]} ^ lut_reg_out_bus;

  assign data_out = data_out_bus[7];
  assign crc_reg_out = ~crc_reg_out_bus[31]; // invert the output
endmodule
