/* CRC32
 *  by Grant Hernandez (Spring 15)
 */

`timescale 10us/1us
module crc32_tb;
  // control signals
  reg clk, reset;
  reg crc_reg_ld, crc_reg_wr, lut_reg_wr, data_wr;
  // data path
  reg [7:0] data_in;
  reg crc_reg_in, lut_reg_in;

  // module output
  wire data_out;
  wire crc_reg_out;
  wire [7:0] lut_addr;
  reg [5:0] lutIdx;

  crc32 dut(
    .clk(clk), .reset(reset),
    .data_in(data_in),
    .data_wr(data_wr),
    .crc_reg_in(crc_reg_in),
    .crc_reg_ld(crc_reg_ld),
    .crc_reg_wr(crc_reg_wr),
    .lut_reg_in(lut_reg_in),
    .lut_reg_wr(lut_reg_wr),

    .data_out(data_out),
    .crc_reg_out(crc_reg_out),
    .lut_addr(lut_addr)
  );

  // testbench state
  integer i, file, dummy;
  integer clkCnt;

  reg [31:0] crc32_tab[0:255];
  reg start;
  integer dataCnt;
  reg [7:0] testString[0:3];
  reg [2:0] dataIdx;

  reg [2:0] state;

  reg [31:0] crc_result;
  reg [5:0] crcIdx;

  initial begin
    $display ("CRC32 Testbench Starting...");
    $dumpvars(0,crc32_tb);

    // read in the crc32 table from a file
    file = $fopen("crc32tab.txt", "r");

    if (!file) begin
      $display("Failed to open crc32 table for reading");
      $finish;
    end

    for(i = 0; i < 256; i = i + 1) begin
      dummy = $fscanf(file, "%h\n", crc32_tab[i]);
      //$display("%H", crc32_tab[i]);
    end

    $fclose(file);

    clkCnt = 0;
    dataCnt = 0;
    lutIdx = 0;
    state = 0;
    crc_result =0;
    crcIdx = 0;

    // DUT init
    crc_reg_ld = 0;
    crc_reg_wr = 0;
    lut_reg_wr = 0;
    data_wr = 0;
    data_in = 0;
    crc_reg_in = 0;
    lut_reg_in = 0;

    clk = 0;
    start = 0;
    reset = 1;
    #2;
    reset = 0;
    #2;
    reset = 1;
    #1;
    start = 1;

    dataIdx = 0;
    testString[0] = 8'h41;
    testString[1] = 8'h42;
    testString[2] = 8'h43;
    testString[3] = 8'h44;

    $display("Using testing string of %h%h%h%h",
      testString[0],
      testString[1],
      testString[2],
      testString[3],
    );
  end

  always clk = #1 start ? ~clk : clk;

  always @(posedge clk) begin
    clkCnt = clkCnt + 1;
    if (clkCnt > 200) begin
      $display("Clock limit reached");
      $finish;
    end
  end

  parameter LOAD_DATA = 0;
  parameter LOOKUP_LUT = 1;
  parameter XOR_MAIN = 2;

  always @(posedge clk) begin
    if (dataIdx == 4) begin
      if (!crcIdx) begin
        $display("CRC32 finished. Clocking out result");
      end

      if (crcIdx == 32) begin
        $display("CRC32 result %h. Cycles %d", crc_result, clkCnt);
        $finish;
      end
      else begin
        crc_result[31-crcIdx] = crc_reg_out;
        crc_reg_in = 1;
        crc_reg_wr = 1;
        crc_reg_ld = 0;
        crcIdx = crcIdx + 1;
      end
    end
    else begin
      case (state)
        LOAD_DATA: begin
          crc_reg_ld = 0;
          crc_reg_wr = 0;

          data_wr = 1;
          data_in = testString[dataIdx];
          lutIdx = 0;
          $display("Loading data %h", testString[dataIdx]);

          // transition in one clk cycle
          state = LOOKUP_LUT;
        end
        LOOKUP_LUT: begin
          data_wr = 0;
          lut_reg_wr = 1;

          if (lutIdx == 32) begin
            $display("Loaded LUT with %h (addr %h)", crc32_tab[lut_addr], lut_addr);
            state = XOR_MAIN;
            lut_reg_wr = 0;
            lut_reg_in = 0;
          end
          else begin
            lut_reg_in = crc32_tab[lut_addr][31-lutIdx];
            lutIdx = lutIdx + 1;
          end
        end
        XOR_MAIN: begin
          crc_reg_ld = 1;
          crc_reg_wr = 1;
          $display("crc_reg %h", dut.crc_reg_out_bus);


          dataIdx = dataIdx + 1;
          state = LOAD_DATA;
        end
        default: begin
          $display("Error: state machine invalid");
          $finish;
        end
      endcase
    end
  end
endmodule
